
void RecoPID(){

  bool printGif = true;

  gStyle->SetOptStat(111111);
  gSystem->Load("libXMLParser.so");
  gSystem->Load("libanacore.so");
  gSystem->Load("libanabrips.so");

  TArtEventStore *estore = new TArtEventStore();

  TArtBigRIPSParameters *setup = new TArtBigRIPSParameters();
  setup->LoadParameter("db/PPAC.xml");
  setup->LoadParameter("db/Plastic.xml");
  setup->LoadParameter("db/IC.xml");
  setup->LoadParameter("db/FocalPlane.xml");
  setup->SetFocusPosOffset(8,148.5);

  // setting up rips parameters
  TArtRIPSPara * rips3to5 = (TArtRIPSPara *)setup->AddOptMatrix(3,5,"matrix/mat1.mat",7.6841); // f3 - f5
  TArtRIPSPara * rips5to7 = (TArtRIPSPara *)setup->AddOptMatrix(5,7,"matrix/mat2.mat",7.6637); // f5 - f7
  TArtRIPSPara * rips8to9 = (TArtRIPSPara *)setup->AddOptMatrix(8,9,"matrix/F8F9_LargeAccAchr_HighBrho2.mat",6.7718); // f8 - f9
  TArtRIPSPara * rips9to11 = (TArtRIPSPara *)setup->AddOptMatrix(9,11,"matrix/F9F11_LargeAccAchr_HighBrho2.mat",6.7707); // f9 - f11

  // setting up tof
  TArtTOFPara * tof3to7 = (TArtTOFPara *)setup->AddTOF("F3pl","F7pl",242.30);
  TArtTOFPara * tof8to11 = (TArtTOFPara *)setup->AddTOF("F8pl","F11pl-1",197.96);

  // setting up beam id devices
  TArtBeamPara *beamp_br = setup->AddBeam(rips3to5,rips5to7,tof3to7,"F7IC");
  TArtBeamPara *beamp_zd = setup->AddBeam(rips8to9,rips9to11,tof8to11,"F11IC");

  TArtRawEventObject *rawevent = new TArtRawEventObject();
  estore->Initialize(rawevent); 
  estore->Open("test.ridf");
 
  TArtCalibPID *mycalib= new TArtCalibPID(setup);
  TArtPID *mypid= new TArtPID(setup);
  mypid->LoadDetectorMap(mycalib);

  TArtFocalPlaneArray *fpl_array = mypid->GetFocalPlane();
  TArtRIPSArray *rips_array = mypid->GetRIPS();
  TArtTOFArray *tof_array = mypid->GetTOF();
  TArtBeamArray *beam_array = mypid->GetBeam();  

  // prepare monitoring histograms
  TFile *fout = new TFile("ntp.root","RECREATE");
  TNtuple *ntp_br = new TNtuple("br","bigrips","aoq:z");
  TNtuple *ntp_zd = new TNtuple("zd","zerodegree","aoq:z");

  char dummy[64];
  TH2 * hFplPos[20];
  TH2 * hFplAngle[20];
  TH1 * hDelta[6];
  TH1 * hTOF[6];
  TH1 * hBeta[6];
  TH2 * hAoQZ[6]; 

  for(int i=0;i<fpl_array->GetEntries();i++) {
    TArtFocalPlane * myfpl = fpl_array->At(i);
    sprintf(dummy,"pos_fpl%d",myfpl->GetFpl());
    hFplPos[i] = new TH2F(dummy,dummy,100,-400,400,100,-400,400);
    sprintf(dummy,"angle_fpl%d",myfpl->GetFpl());
    hFplAngle[i] = new TH2F(dummy,dummy,100,-400,400,100,-400,400);
  }
  for(int i=0;i<rips_array->GetEntries();i++) {
    TArtRIPS * myrips = rips_array->At(i);
    sprintf(dummy,"rips_delta_f%d_b%d",
	    myrips->GetForwardFpl(),myrips->GetBackwardFpl());
    hDelta[i] = new TH1F(dummy,dummy,100,-10,10);
  }
  for(int i=0;i<tof_array->GetEntries();i++) {
    TArtTOF * mytof = tof_array->At(i);
    sprintf(dummy,"tof_f%s_b%s",
	    mytof->GetForwardPlName()->Data(),
	    mytof->GetBackwardPlName()->Data());
    hTOF[i] = new TH1F(dummy,dummy,100,-300,300);
    sprintf(dummy,"beta_f%s_b%s",
	    mytof->GetForwardPlName()->Data(),
	    mytof->GetBackwardPlName()->Data());
    hBeta[i] = new TH1F(dummy,dummy,100,-0.5,1.5);
  }
  for(int i=0;i<beam_array->GetEntries();i++) {
    TArtBeam * mybeam = beam_array->At(i);
    sprintf(dummy,"aoqz_f%d_b%d",
	    mybeam->GetForwardRIPS()->GetForwardFpl(),
	    mybeam->GetBackwardRIPS()->GetBackwardFpl());
    hAoQZ[i] = new TH2F(dummy,dummy,100,-100,100,110,-10,100);
  }

  std::cout << "start to scan" << std::endl;

  int neve = 0;
  while(estore->GetNextEvent() && neve<10000){
    mycalib->ReconstructData();
    mypid->ReconstructData(mycalib);
    TArtBeam *beam_br = mypid->GetBeam(beamp_br);
    TArtBeam *beam_zd = mypid->GetBeam(beamp_zd);
    ntp_br->Fill(beam_br->GetAoQ(),beam_br->GetZet());
    ntp_zd->Fill(beam_zd->GetAoQ(),beam_zd->GetZet());

    for(int i=0;i<fpl_array->GetEntries();i++){
      TVectorD * optvec = ((TArtFocalPlane*)fpl_array->At(i))->GetOptVector();
      hFplPos[i]->Fill(optvec(0),optvec(2));
      hFplAngle[i]->Fill(optvec(1),optvec(3));
    }
    for(int i=0;i<rips_array->GetEntries();i++){
      TArtRIPS * myrips = (TArtRIPS*)rips_array->At(i);
      hDelta[i]->Fill(myrips->GetDelta());
    }
    for(int i=0;i<tof_array->GetEntries();i++){
      TArtTOF * mytof = (TArtTOF*)tof_array->At(i);
      hTOF[i]->Fill(mytof->GetTOF());
      hBeta[i]->Fill(mytof->GetBeta());
    }
    for(int i=0;i<beam_array->GetEntries();i++){
      TArtBeam * mybeam = (TArtBeam*)beam_array->At(i);
      hAoQZ[i]->Fill(mybeam->GetAoQ(),mybeam->GetZet());
    }
    mycalib->ClearData();
    mypid->ClearData();
    rawevent->Clear();
    neve ++;
  }

  std::cout << "finish to scan" << std::endl;

  // histograms for FocalPlane
  TCanvas *cfpl = new TCanvas("cfpl","cfpl",1600,1200);
  cfpl->Divide(4,3);
  for(int i=0;i<fpl_array->GetEntries();i++) {
    cfpl->cd(i+1); hFplPos[i]->Draw("colz");
  }
  if(printGif)cfpl->Print("cfpl.gif");

  TCanvas *cfpla = new TCanvas("cfpla","cfpla",1600,1200);
  cfpla->Divide(4,3);
  for(int i=0;i<fpl_array->GetEntries();i++) {
    cfpla->cd(i+1); hFplAngle[i]->Draw("colz");
  }
  if(printGif)cfpla->Print("cfpla.gif");

  // histograms for RIPS

  TCanvas *cdelta = new TCanvas("cdelta","cdelta",1600,1200);
  cdelta->Divide(3,2);
  for(int i=0;i<rips_array->GetEntries();i++) {
    cdelta->cd(i+1); hDelta[i]->Draw();
  }
  if(printGif)cdelta->Print("cdelta.gif");

  TCanvas *ctof = new TCanvas("ctof","ctof",1600,1200);
  ctof->Divide(3,2);
  for(int i=0;i<tof_array->GetEntries();i++) {
    ctof->cd(i+1); hTOF[i]->Draw();
  }
  if(printGif)ctof->Print("ctof.gif");

  TCanvas *cbeta = new TCanvas("cbeta","cbeta",1600,1200);
  cbeta->Divide(3,2);
  for(int i=0;i<tof_array->GetEntries();i++) {
    cbeta->cd(i+1); hBeta[i]->Draw();
  }
  if(printGif)cbeta->Print("cbeta.gif");

  TCanvas *cbeam = new TCanvas("cbeam","cbeam",1600,1200);
  cbeam->Divide(3,2);
  for(int i=0;i<beam_array->GetEntries();i++) {
    cbeam->cd(i+1); hAoQZ[i]->Draw("colz");
  }
  if(printGif)cbeam->Print("cbeam.gif");

  fout->Write();
  fout->Close();


}
