#ifndef TARTALIAS_HH
#define TARTALIAS_HH

#include "TObject.h"
#include "TFitResult.h"
#include "TFitResultPtr.h"

class TVirtualPad;
class TH1;
class TH1D;
class TH2;
class TProfile;

//Read all TH1 object in file(filename) to current directory.
void fetch(char* filename);
//Write all TH1 object in current directory to file(filename)
void hstore(char* filename);
//Delete all TH1 object in current directory.
void clear();

//show list of hist in current directory.
void ls();

//move to nexd pad and draw hist from idstart to idend
void ht(int idstart, int idend);
//move to nexd pad and draw hist by id.
void ht(int id);
//move to nexd pad and draw current hist.
void htp();
//move to nexd pad and draw next hist.
void hn();
//move to nexd pad and draw previous hist.
void hb();

//draw hist from idstart to idend
void hht(int idstart, int idend);
//draw hist by id.
void hht(int id);
//draw current hist.
void hhtp();
//draw next hist.
void hhn();
//draw previous hist.
void hhb();

//set xaxis of gPad to logscale
void lgx();
//set yaxis of gPad to logscale
void lgy();
//set zaxis of gPad to logscale
void lgz();
//unset lgx
void lnx();
//unset lgy
void lny();
//unset lgz
void lnz();
//do lgx, lgy, and lgz
void lg();
//do lnx, lny, and lnz
void ln();

//change size of canvas
void size(UInt_t ww, UInt_t wh);
//cd to subpadnumber(0 is assigned to root canvas)
void cd(Int_t subpadnumber = 0);
//cd to next subpad
void cdn();
//cd to previous subpad
void cdb();
//divide canvas(not gPad).
void zone(Int_t nx = 1, Int_t ny = 1, Float_t xmargin = 0.01, Float_t ymargin = 0.01, Int_t color = 0);

//gui command
Double_t xval();
Double_t yval();
void xyval();
void xzoom();
void yzoom();
void xyzoom();
void xblow();
void yblow();
void xyblow();
//cut out selected region of TH2
void cpro();

//same as fitg()
void figa();
TFitResultPtr fit(const char* formula, Option_t* option = "", Option_t* goption = "", Double_t xmin = 0, Double_t xmax = 0);
TFitResultPtr fitp(int n = 1, Option_t* option = "", Option_t* goption = "", Double_t xmin = 0, Double_t xmax = 0);
TFitResultPtr fite(Option_t* option = "", Option_t* goption = "", Double_t xmin = 0,  Double_t xmax = 0);
TFitResultPtr fitg(Option_t* option = "", Option_t* goption = "", Double_t xmin = 0,  Double_t xmax = 0);
TFitResultPtr fitl(Option_t* option = "", Option_t* goption = "", Double_t xmin = 0,  Double_t xmax = 0);

//profile
void pfx();
void pfy();
//projection
void prx();
void pry();
//ban
void bnx(double firstx, double lastx, Option_t* option = "");
void bny(double firsty, double lasty, Option_t* option = "");
//zoom
void zoom(double firstx = 0, double lastx = -1,
     double firsty = 0, double lasty = -1,
     double firstz = 0, double lastz = -1);
void zoomx(double firstx = 0, double lastx = -1);
void zoomy(double firsty = 0, double lasty = -1);
void zoomz(double firstz = 0, double lastz = -1);
//blow
void blow(double firstx = 0, double lastx = -1,
     double firsty = 0, double lasty = -1,
     double firstz = 0, double lastz = -1);
void blowx(double firstx = 0, double lastx = -1);
void blowy(double firsty = 0, double lasty = -1);
void blowz(double firstz = 0, double lastz = -1);

//min max
void mami(double min, double max);

#endif
