#ifndef _TARTTOF_H_
#define _TARTTOF_H_

#include <TObjArray.h>
#include <TNamed.h>

class TArtBigRIPSParameters;
class TArtPlastic;
class TArtPlasticArray;
class TArtTOFPara;

class TArtTOFArray : public TObjArray {

 public:
  TArtTOFArray() :loadplmap(false){}
  TArtTOFArray(TArtBigRIPSParameters *para) :loadplmap(false){LoadSetup(para);}
  virtual ~TArtTOFArray(){}

  virtual void LoadSetup(TArtBigRIPSParameters *para);
  virtual void LoadPlasticMap(TArtPlasticArray *pls);
  virtual void ClearData();
  virtual void RecoTOF(); // reconstruct delta

  virtual Bool_t IsPlasticMapLoaded(){return loadplmap;}

 private:
  Bool_t loadplmap;

  ClassDef(TArtTOFArray,1);

};

class TArtTOF : public TNamed
{
 public: 

  TArtTOF(TArtTOFPara *para);
  virtual ~TArtTOF();

  virtual void RecoTOF();
  virtual void ClearData();
  virtual void LoadPlasticMap(TArtPlasticArray *pls);

  double GetTOF(){return tof;}
  double GetBeta(){return beta;}
  double GetGamma(){return gamma;}

  // calibration parameter
  Int_t GetID();
  TString * GetForwardPlName();
  TString * GetBackwardPlName();

 protected:

  // time of flight
  double tof;
  // speed of particle
  double beta, gamma;
  // speed of light
  double clight;

  // length for tof
  double tof_length;

  //setup
  TArtTOFPara * para;
  // forward plastic scintillator 
  TArtPlastic * forward_pla;
  // backward plastic scintillator 
  TArtPlastic * backward_pla;

  ClassDef(TArtTOF,1);

};

#endif
