#include "TArtCalibDALI.h" 
#include "TArtDALI.h"
#include "TArtDALIPara.h"
#include "TArtDALIParameters.h"
#include "TArtRawEventObject.h"
#include "TArtCore.h" 
#include "TArtMapper.h"

#include <TROOT.h>

//__________________________________________________________
TArtCalibDALI::TArtCalibDALI()
  :fReconstructed(false)
{
  TArtCore::Info(__FILE__,"Creating the DALI detector objects...");
  setup = (TArtDALIParameters*)gROOT->FindObject("DALIParameters");
  fDALI = new TArtDALIArray();
  fDALI->SetOwner();
}

//__________________________________________________________
TArtCalibDALI::TArtCalibDALI(TArtDALIParameters * s)
  :fReconstructed(false), setup(s)
{
  TArtCore::Info(__FILE__,"Creating the DALI detector objects...");
  fDALI = new TArtDALIArray();
  fDALI->SetOwner();
}

//__________________________________________________________
TArtCalibDALI::~TArtCalibDALI()  {
  fDALI->Delete();
}

//__________________________________________________________
void TArtCalibDALI::LoadRawData(TArtRawEventObject *fEvent)   {

  ClearData();
  for(int i=0;i<fEvent->GetNumSeg();i++){
    TArtRawSegmentObject *seg = fEvent->GetSegment(i);
    int device   = seg->GetDevice();
    int fpl      = seg->GetFP();
    int detector = seg->GetDetector();
    if(DALI == device)
      for(int j=0;j<seg->GetNumData();j++){
	TArtRawDataObject *d = seg->GetData(j);
	int geo = d->GetGeo(); 
	int ch = d->GetCh(); 
	unsigned int val = d->GetVal();
	TArtRIDFMap mm(fpl,detector,geo,ch);
	TArtDALIPara *para = (TArtDALIPara*)setup->GetDALIPara(&mm);
	if(NULL == para){
	  TArtCore::Info(__FILE__,"Could not find TArtDALIPara...: Dev:%d, %s", device, mm.GetMapInfo());
	  continue;
	}

	int id = para->GetID();
	int nnai = fDALI->GetEntries();
	TArtCore::Debug(__FILE__,"Entries in DALIArray: %d, DALI-ID:%d", nnai, id);
	TArtDALI * nai = fDALI->FindNaI(id);

	if(nai==NULL){
	  new ((*fDALI)[nnai]) TArtDALI(para);
	  nai = (TArtDALI *)fDALI->At(nnai);
	}

	// set raw data
	if(DALIA == detector)
	  nai->SetRawADC(val);
	if(DALIT == detector)
	  nai->SetRawTDC(val);

      }
  }
  return;

}

//__________________________________________________________
void TArtCalibDALI::ClearData()   {

  (*fDALI).Delete();
  fReconstructed = false;
  return;

}

void TArtCalibDALI::ReconstructData(TArtRawEventObject *fEvent)   {

  LoadRawData(fEvent);
  ReconstructData();

  return;
}

//__________________________________________________________
void TArtCalibDALI::ReconstructData()   { // call after the raw data are loaded

  fDALI->ReconstructData();
  fReconstructed = true;
  return;

}

//__________________________________________________________
TArtDALI * TArtCalibDALI::GetDALI(int i) {
  return (TArtDALI *)fDALI->At(i);
}
//__________________________________________________________
Int_t TArtCalibDALI::GetNumDALI() {
  return fDALI->GetEntries();
}
